#!/usr/bin/perl

package IPAnalyze::Taint;
use strict;

sub new {
   my $me = shift;
   return bless({}, $me);
}

sub unknown_parents {
   my $self = shift;
   my $dbh = shift || die "unknown_parents syntax error1\n";
   my $ipmath = shift || die "unknown_parents syntax error2\n";

	my (%netblocks,@errors);
   my $sql = 'SELECT id,netblock,cidr FROM parent_networks';
   my $sth = $dbh->prepare($sql);
   $sth->execute;
   while(my $row = $sth->fetchrow_arrayref){
		my @netblocks = $ipmath->get_netblocks($row->[1],$row->[2],30);
		$netblocks{$row->[0]} = \@netblocks;
	}
	
	$sql = 'SELECT netid,netblock,cidr FROM data WHERE parent_id = 0 AND netblock IS NOT NULL';
	$sth = $dbh->prepare($sql);
	$sth->execute;
	while(my $row = $sth->fetchrow_arrayref){
		# is netblock/cidr inside any parent that we already have?
		foreach my $id (keys %netblocks){
			if(grep {/^$row->[1]$/} @{$netblocks{$id}}){
				# found a match
				my $sqla = 'UPDATE data SET parent_id = ' . $dbh->quote($id);
				$sqla .= ' WHERE netid = ' . $dbh->quote($row->[0]);
				my $stha = $dbh->prepare($sqla);
				$stha->execute;
				last;
			}
		}
	}
	return(@errors);
}

sub taint_parents {
   my $self = shift;
   my $dbh = shift || die "taint_parents syntax error1\n";
   my $ipmath = shift || die "taint_parents syntax error2\n";

   my (%gp,%p,@errors);
   my $sql = 'SELECT id,netblock,cidr FROM grandparent_networks';
   my $sth = $dbh->prepare($sql);
   $sth->execute;
   while(my $row = $sth->fetchrow_arrayref){
		$gp{$row->[0]} = [$row->[1], $row->[2]];

      my $supernet = $ipmath->get_netblock($row->[1],$row->[2]);
      unless($supernet eq $row->[1]){
         push @errors, "Grandparent id $row->[0] inconsistent!";
      }
   }

   $sql = 'SELECT id,parent_id,netblock,cidr FROM parent_networks';
   $sth = $dbh->prepare($sql);
   $sth->execute;
   while(my $row = $sth->fetchrow_arrayref){
      # make sure this netblock/cidr is in grandparent's netblock/cidr
		$p{$row->[0]} = [$row->[2],$row->[3]];

		if(exists($gp{$row->[1]})){
			my ($pnetblock,$pcidr) = @{$gp{$row->[1]}}[0,1];
      	my @netblocks = $ipmath->get_netblocks($pnetblock,$pcidr,$row->[3]);
    		unless(grep {/^$row->[2]$/} @netblocks){
				push @errors, "Parent id $row->[0] is not part of grandparent $row->[1]\n";

				my $found;
				while(my($id,$arrayref) = each %gp){
					my @netblocks = $ipmath->get_netblocks($arrayref->[0],$arrayref->[1],$row->[3]);
					if(grep {/^$row->[2]$/} @netblocks){
						
						push @errors, "CORRECTED grandparent_id for parent_id $row->[0]: $id\n";

						my $sqla = 'UPDATE parent_networks SET parent_id = ' . $dbh->quote($id);
						$sqla .= ' WHERE id = ' . $dbh->quote($row->[0]);
   					my $stha = $dbh->prepare($sqla);
						$stha->execute;

						$found=1;
						last;
					}
				}
				unless($found){
					push @errors, "COULD NOT find grandparent_id for parent_id $row->[0]\n";
				}
			}
		}else{	
			push @errors, "Parent id $row->[0] has no existing grandparent id $row->[1]\n";
		}
   }

	# make sure we dont have a sub-parent of a parent (10.0.0.0/22 and 10.0.2.0/24) - warn about 10.0.2.0
#	foreach my $id (keys %p){
#		my($netblock,$cidr) = @{$p{$id}}[0,1];
#      
#		warn "id: $id - netblock: $netblock - cidr: $cidr\n";
#		foreach my $otherid (keys %p){
#			my($othernetblock,$othercidr) = @{$p{$otherid}}[0,1];
#			# if this one fits in the one outside this loop, warn
#
#			warn "otherid: $otherid - othernetblock: $othernetblock - othercidr: $othercidr\n";
#
#			my @netblocks = $ipmath->get_netblocks($netblock,$cidr,$othercidr);
#			foreach my $netblock (@netblocks){
#				next if(($netblock eq $othernetblock) && ($cidr eq $othercidr));
#				if($netblock eq $othernetblock){
#					push @errors, "CONFLICT id: $id - network: ${netblock}/${cidr} - otherid: $otherid - othernetwork: ${othernetblock}/${othercidr}";
#				}
#			}
#		}
#	}

	$sql = 'SELECT netid,parent_id,netblock,cidr FROM data WHERE netblock IS NOT NULL';
	$sql .= ' AND parent_id != 0'; #QQQ
   $sth = $dbh->prepare($sql);
   $sth->execute;
   while(my $row = $sth->fetchrow_arrayref){
      # make sure this netblock/cidr is in parent's netblock/cidr
	
		if(exists($p{$row->[1]})){
			my ($pnetblock,$pcidr) = @{$p{$row->[1]}}[0,1];
   	   my @netblocks = $ipmath->get_netblocks($pnetblock,$pcidr,$row->[3]);
    		unless(grep {/^$row->[2]$/} @netblocks){
				push @errors, "Netid $row->[0] ($row->[2]/$row->[3]) is not part of parent $row->[1] " .
				              "($pnetblock/$pcidr)";

				my $found;
				while(my($id,$arrayref) = each %p){
					my @netblocks = $ipmath->get_netblocks($arrayref->[0],$arrayref->[1],$row->[3]);
					if(grep {/^$row->[2]$/} @netblocks){
						
						push @errors, "CORRECTED parent_id for netid $row->[0]: $id\n";

						my $sqla = 'UPDATE data SET parent_id = ' . $dbh->quote($id);
						$sqla .= ' WHERE netid = ' . $dbh->quote($row->[0]);
   					my $stha = $dbh->prepare($sqla);
						$stha->execute;

						$found=1;
						last;
					}
				}
				unless($found){
					my $sqla = 'UPDATE data SET parent_id = 0 WHERE netid = ' . $dbh->quote($row->[0]);
   				my $stha = $dbh->prepare($sqla);
					$stha->execute;
					push @errors, "COULD NOT find parent_id for netid $row->[0]\n";
				}
			}
		}else{
			push @errors, "Netid $row->[0] has no existing parent id $row->[1]\n";
		}
	}

   return(@errors);
}

1;
