#!/usr/local/bin/perl

# to be ran nightly (or more) to expire requests that have been
# waiting too long or revoked
#
# also assigns requests that are still waiting, but area already in IGP

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use Net::SMTP;
use Sys::Hostname;
use IPAnalyze::Misc;
use IPAnalyze::IPmath;

my $hostname = hostname();

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open ipanalyze.conf: $!\n";
while(<CONFIG>){
   s/^(.*)\s*#.*/$1/;
   next if(/^$/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $misc = IPAnalyze::Misc->new();
my $ipmath = IPAnalyze::IPmath->new();

my $dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
my $radius = DBI->connect('DBI:mysql:host=mysql.example.com;database=isp', 'root', undef, {RaiseError => 1});

my $stuff = 'netid,circuit_id,netblock,cidr,status,local_email';
my $sql = "SELECT $stuff FROM data WHERE status < 5 AND netblock IS NOT NULL";
$sql .= ' AND status != 0 ORDER BY bornondate';
my $sth = $dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){

	my $code = 1; # if netblock is not allocated yet (sitting in allocate, etc)
	if($row->[2]){
		# if netblock is allocated
		$code = $misc->check_igp($ipmath,$cfg{'igp_host'},$cfg{'igp_community'},$row->[2],$row->[3],$cfg{'igp_ignore_netmask'});
	}
	unless($code){
		warn "$row->[0] - $row->[2]/$row->[3] is in IGP, but IPAnalyze says it's NOT ACTIVE - Correcting\n";

		# assume it's good and someoen forgot to assign it.
		my $notes = $misc->get_notes($dbh,$row->[0]);
		$notes .= "\n\nAssigned by AutoAssign (found route in IGP)";
		my $sqlb = 'UPDATE data SET status = 0, notes = ' . $dbh->quote($notes);
		$sqlb .= ' WHERE netid = ' . $dbh->quote($row->[0]);
		my $sthb = $dbh->prepare($sqlb);
		$sthb->execute;

	}
}

