#!/usr/local/bin/perl

# to be ran nightly (or more) to generate rwhois data.
# also makes new private-id
# assumes:
#  you have set up rwhois via http://www.unixadmin.cc/rwhois/rwhois-ip-howto.html
#  the rwhois user's home dir is the directory where you installed rwhois
#  all networks in grandparent_networks are completely managed by ipanalyze

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use Date::Calc qw(Today);
use IPAnalyze::Misc;
use IPAnalyze::IPmath;
use IPAnalyze::Customers;

my $DEBUG=1;
my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open ipanalyze.conf: $!\n";
while(<CONFIG>){
   s/^(.*)\s*#.*/$1/;
   next if(/^$/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

chop(my $date=`date '+%Y%m%d'`);
my $full = ($ARGV[0] eq '-f') ? 1 : 0;

my $homedir = (getpwnam($cfg{rwhois_user}))[7];
if($homedir){
	if(-d $homedir){
		chdir($homedir) || die "cannot chdir $homedir: $!\n";
	}else{
		die "$cfg{rwhois_user}'s homedir does not exist: $homedir\n";
	}
}else{
	die "cannot determine homedir of rwhois_user: $cfg{rwhois_user}\n";
}

my $our_name = lc($cfg{our_name});
$our_name =~ s/\s//g;
my $ucour_name = uc($our_name);
my $now = $^T;

my $restart;
my $d_dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
my $c_dbh = $d_dbh;
if($cfg{c_dsn} && $cfg{c_dbun} && $cfg{c_dbpw}){
	unless(($cfg{d_dsn} eq $cfg{c_dsn}) && ($cfg{d_dbun} eq $cfg{c_dbun}) && ($cfg{d_dbpw} eq $cfg{c_dbpw})){
		$c_dbh = DBI->connect($cfg{c_dsn}, $cfg{c_dbun}, $cfg{c_dbpw}, {RaiseError => 1});
	}
}
my $misc = IPAnalyze::Misc->new();
my $ipmath = IPAnalyze::IPmath->new();
my $customers = IPAnalyze::Customers->new();

# nuke all existing private_ids
opendir(D, '/export/home/vpopmail/domains/example.com/') || die "cannot open domains/example.com: $!\n";
foreach my $file (grep {/^\.qmail-prwhois-[a-z0-9_:]{32}$/} readdir D){
	unlink("/export/home/vpopmail/domains/example.com/$file") || warn "couldnt unlink example.com/$file: $!\n";
}
closedir D;

# give all private custs new private_ids
my $sql = 'SELECT netid,tech_email,netblock FROM data WHERE private_id IS NOT NULL';
my $sth = $d_dbh->prepare($sql);
$sth->execute;
while(my $row=$sth->fetchrow_arrayref){
	next if($row->[2] =~ /^(?:172\.(?:1[6789]|2\d|3[01])\.|10\.|192\.168\.)/);
	my $new_pid = $misc->get_rand(32);
	$new_pid =~ tr/A-Z/a-z/;
	my $file = $new_pid;
	$file =~ s/\./:/g;
	if(open(F, ">/export/home/vpopmail/domains/example.com/.qmail-prwhois-$file")){
		print F '&', $row->[1], "\n";
		close F;
		my $sqla = 'UPDATE data SET private_id = ' . $d_dbh->quote($new_pid);
		$sqla .= ' WHERE netid = ' . $d_dbh->quote($row->[0]);
		my $stha = $d_dbh->prepare($sqla);
		$stha->execute;
	}else{
		warn "couldnt create example.com/.qmail-prwhois-$row->[1]: $!\n";
	}
}


# read which auth areas are configured
my %auth_area;
open(R, "${homedir}/rwhoisd.auth_area") || die "cannot read auth_area: $!\n";
open(W, ">${homedir}/rwhoisd.auth_area.tmp") || die "cannot write auth_area.tmp: $!\n";
while(<R>){
	if(/^name:\s*((\d{1,3}\.){3}\d{1,3}\/\d{2})/){
		$auth_area{$1} = 1;
	}
	print W;
}
close R;

$sql = 'SELECT id,netblock,cidr FROM grandparent_networks';
$sth = $d_dbh->prepare($sql);
$sth->execute;
my (%netblocks,@directories);
while(my $row = $sth->fetchrow_arrayref){
	my $id = $row->[0];
	my $netblock = $row->[1];
	my $cidr = $row->[2];
	
	next if($netblock =~ /^(?:172\.(?:1[6789]|2\d|3[01])\.|10\.|192\.168\.)/);

	push @directories, "${netblock}-${cidr}"; # we nuke everything in here later

	$netblocks{$row->[0]} = "$row->[1]/$row->[2]";
	warn "DEBUG: netblocks{$row->[0]} = $row->[1]/$row->[2]\n" if($DEBUG);

	my($oct1,$oct2,$oct3,$oct4) = split(/\./, $netblock);
	unless(exists($auth_area{"$netblock/$cidr"})){
		print W "type: master\n",
		        "name: ${netblock}/${cidr}\n",
		        "data-dir: net-${oct1}.${oct2}.${oct3}.${oct4}-${cidr}/data\n",
		        "schema-file: net-${oct1}.${oct2}.${oct3}.${oct4}-${cidr}/schema\n",
		        "soa-file: net-${oct1}.${oct2}.${oct3}.${oct4}-${cidr}/soa\n",
		        "---\n";
		$auth_area{"$netblock/$cidr"} = 1;
		$restart=1;
	}
	# build the directory structure properly for each network
	unless(-d "${homedir}/net-${netblock}-${cidr}"){
		mkdir("${homedir}/net-${netblock}-${cidr}",0755) || die "cannot mkdir for $netblock/$cidr: $!\n";
		mkdir("${homedir}/net-${netblock}-${cidr}/attribute_defs",0755) || die "cannot mkdir for defs $netblock/$cidr: $!\n";
		mkdir("${homedir}/net-${netblock}-${cidr}/data",0755) || die "cannot mkdir for data $netblock/$cidr: $!\n";
		mkdir("${homedir}/net-${netblock}-${cidr}/data/network",0755) || die "cannot mkdir for network $netblock/$cidr: $!\n";
		mkdir("${homedir}/net-${netblock}-${cidr}/data/org",0755) || die "cannot mkdir for org $netblock/$cidr: $!\n";
		mkdir("${homedir}/net-${netblock}-${cidr}/data/referral",0755) || die "cannot mkdir for ref $netblock/$cidr: $!\n";
		$restart=1;
	}

	open(SOA, ">${homedir}/net-${netblock}-${cidr}/soa") || die "cannot write to soa for $netblock/$cidr: $!\n";
	print SOA "Serial-Number: ${date}01\n",
	          "Refresh-Interval: 3600\n",
	          "Increment-Interval: 1800\n",
	          "Retry-Interval: 60\n",
	          "Time-To-Live: 86400\n",
	          "Primary-Server: $cfg{rwhois_primary}\n",
	          "Hostmaster: $cfg{rwhois_hostmaster}\n";
	close SOA;

	open(SCHEMA, ">${homedir}/net-${netblock}-${cidr}/schema") || die "cannot write to schema for $netblock/$cidr: $!\n";
	print SCHEMA "name: network\n",
	             "attributedef: net-${netblock}-${cidr}/attribute_defs/network.tmpl\n",
	             "dbdir: net-${netblock}-${cidr}/data/network\n",
	             "Schema-Version: ${date}01\n",
	             "---\n",
	             "name: organization\n",
	             "attributedef: net-${netblock}-${cidr}/attribute_defs/org.tmpl\n",
	             "dbdir: net-${netblock}-${cidr}/data/org\n",
	             "Schema-Version: ${date}01\n",
	             "---\n",
	             "name: referral\n",
	             "attributedef: net-${netblock}-${cidr}/attribute_defs/referral.tmpl\n",
	             "dbdir: net-${netblock}-${cidr}/data/referral\n",
	             "Schema-Version: ${date}01\n";
	close SCHEMA;

	open(F, "${homedir}/etc/rwhoisd/samples/a.com/attribute_defs/org.tmpl") || die "cannot open org.tmpl: $!\n";
	open(ORG, ">${homedir}/net-${netblock}-${cidr}/attribute_defs/org.tmpl") || die "cannot write org.tmpl: $!\n";
	while(<F>){
		if(/attribute:\s*Phone/){
			print ORG;
			while(<F>){
				if(/required:\s*TRUE/){
					print ORG "is-required:     FALSE\n";
				}else{
					print ORG;
				}
			}
		}
		print ORG;
	}
	close ORG;
	close F;
	opendir(D, "${homedir}/etc/rwhoisd/samples/net-10.0.0.0-8/attribute_defs/") || die "cannot opendir attribute_defs: $!\n";
	foreach my $file (grep {!/^\./} readdir D){
		next if($file eq 'org.tmpl');
		open(R, "${homedir}/etc/rwhoisd/samples/net-10.0.0.0-8/attribute_defs/${file}") || die "cannot read defs/${file}: $!\n";
		open(F, ">${homedir}/net-${netblock}-${cidr}/attribute_defs/${file}") || die "cannot write to defs/${file}: $!\n";
		while(<R>){
			if($file eq 'network.tmpl' && /attribute:\s*IP-Network$/){
				print F;
				while(<R>){
					if(/^---/){
						print F "---\n",
						        "attribute:       IP-Network-Block\n",
						        "attribute-alias: IPNB\n",
						        "description:     IP address range\n",
						        "is-primary-key:  FALSE\n",
						        "is-required:     TRUE\n",
						        "is-repeatable:   FALSE\n",
						        "is-multi-line:   FALSE\n",
						        "is-hierarchical: FALSE\n",
						        "index:           NONE\n",
						        "type:            TEXT\n",
						        "---\n";
						last;
					}else{
						print F;
					}
				}
			}else{
				print F;
			}
		}
		close F;
		close R;
	}
	closedir D;

	open(F, ">${homedir}/net-${netblock}-${cidr}/data/org/${our_name}.txt") || die "cannot write org/${our_name}.txt: $!\n";
	print F "ID: NETBLCK-${ucour_name}.${netblock}/${cidr}\n",
	        "Auth-Area: ${netblock}/${cidr}\n",
	        "Org-Name: $cfg{our_name}\n",
	        "Street-Address: $cfg{our_addr}\n",
	        "City: $cfg{our_city}\n",
	        "State: $cfg{our_state}\n",
	        "Postal-Code: $cfg{our_zip}\n",
		     "Country-Code: $cfg{our_country}\n",
	        "Phone: $cfg{our_phone}\n",
	        "Created: 20000630\n",
	        "Updated: ${date}01\n";
	close F;

	open(F, ">${homedir}/net-${netblock}-${cidr}/data/network/${netblock}-${cidr}.txt") ||
	 die "cannot write $netblock-$cidr.txt: $!\n";

	my $broadcast = $ipmath->get_broadcast($netblock,$cidr);

	print F "ID: NETBLCK-${ucour_name}.${netblock}/${cidr}\n",
	        "Auth-Area: ${netblock}/${cidr}\n",
	        "Network-Name: ${ucour_name}-${netblock}\n",
	        "IP-Network: ${netblock}/${cidr}\n",
	        "IP-Network-Block: ${netblock} - ${broadcast}\n",
	        "Organization: $cfg{our_name}\n",
	        "Tech-Contact: $cfg{rwhois_hostmaster}\n",
	        "Admin-Contact: $cfg{rwhois_nic_handle}\n",
	        "Created: 20000630\n",
	        "Updated: ${date}01\n",
	        "Updated-By: $cfg{rwhois_hostmaster}\n";
	close F;
}

close W;

my %gpid; # grand-parent id
$sql = 'SELECT id,parent_id FROM parent_networks';
$sth = $d_dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	$gpid{$row->[0]} = $row->[1];
	warn "DEBUG2: gpid{$row->[0]} = $row->[1]\n" if($DEBUG);
}

# build the networks and orgs
my %built;
my $stuff = 'cust_id,parent_id,netblock,cidr,private_id,tech_email,tech_phone,tech_ext,bornondate';
$sql = "SELECT $stuff FROM data WHERE netblock IS NOT NULL";
$sth = $d_dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_hashref){
	my $cust_id = $row->{cust_id};
	my $parent_id = $row->{parent_id};
	my $netblock = $row->{netblock};
	my $cidr = $row->{cidr};
	my $born = $row->{bornondate};
	my $tech_phone = $row->{tech_phone};
	if($tech_phone eq '0'){
		$tech_phone = $cfg{'our_phone'};
	}else{
		$tech_phone =~ s/(\d+)(\d{3})(\d{4})$/$1-$2-$3/;
	}
	my $tech_ext = $row->{tech_ext};

	next if($netblock =~ /^(?:172\.(?:1[6789]|2\d|3[01])\.|10\.|192\.168\.)/);

	if($parent_id == 0){
		warn "fudging parent_id\n" if($DEBUG);
		if($netblock =~ /^64\.115\./){
			$parent_id = 289; # 64.115.60.0/24
		}elsif($netblock =~ /^64\.61\.([0-9]{1,3})\./){
			my $oct = $1;
			if($oct < 128){
				$parent_id = 439; # 64.61.48.0/21 
			}else{
				$parent_id = 282; # 64.61.136.0/21 
			}
		}else{
			next; # too hard to fudge
		}
	}

	my $netdir = $netblocks{$gpid{$parent_id}};
	$netdir =~ s#/#-#;

	# we do a full rebuild once a week, otherwise it's a quick rebuild (via cron)
	unless($full){
		# quick rebuild
		# check to see if this network is already rwhois'd.
		if((-f "${homedir}/net-${netdir}/data/org/${cust_id}.txt") &&
		   (-f "${homedir}/net-${netdir}/data/network/${netblock}-${cidr}.txt")){
			warn "quick mode: skipping ${netblock}/${cidr} for $cust_id\n" if($DEBUG);
			next;
		}else{
			warn "quick mode: NOT SKIPPING ${netblock}/${cidr} for $cust_id\n" if($DEBUG);
		}
	}

	my $cust = $customers->all($c_dbh,$d_dbh,'cust_id',$row->{cust_id});
	my $lcname = lc($cust->[1]);
	$lcname =~ s/\s/-/g;
	my $ucname = uc($lcname);

	my ($day,$mon,$year) = (localtime($born))[3,4,5];
	$day = sprintf("%02d", $day);
	$mon = sprintf("%02d", $mon++);
	$year+= 1900;
	my $created = "${year}${mon}${day}";

	# build org if needed
	unless(exists($built{$cust_id}{$netdir})){
		warn "cust_id: $cust_id\n",
			"parent_id: $parent_id\n",
			"gpid{$parent_id} = $gpid{$parent_id}\n",
			"netblocks{$gpid{$parent_id}} = $netblocks{$gpid{$parent_id}}\n" if($DEBUG);
		if(open(F, ">${homedir}/net-${netdir}/data/org/${cust_id}.txt.tmp")){
			print F "ID: NETBLCK-${ucour_name}.$netblocks{$gpid{$parent_id}}\n",
					  "Auth-Area: $netblocks{$gpid{$parent_id}}\n",
					  "Org-Name: ";
			if($row->{private_id}){
				print F "$cfg{our_name}: $row->{private_id}\n",
						  "Street-Address: $cfg{our_addr}\n",
						  "City: $cfg{our_city}\n",
						  "State: $cfg{our_state}\n",
						  "Postal-Code: $cfg{our_zip}\n",
						  "Country-Code: $cfg{our_country}\n",
						  "Phone: $cfg{our_phone}\n";
			}else{
				my $country = $cust->[6] || 'US';
				print F "$cust->[1]\n",
						  "Street-Address: $cust->[2]\n",
						  "City: $cust->[3]\n",
						  "State: $cust->[4]\n",
						  "Postal-Code: $cust->[5]\n",
						  "Country-Code: $country\n",
						  "Phone: ${tech_phone}";
				if(defined($tech_ext)){
					print F " x${tech_ext}";
				}
				print F "\n";
			}

			my ($year,$month,$day) = Today($born);
			$month = sprintf("%02d",$month);
			$day = sprintf("%02d",$day);
			print F "Created: ${year}${month}${day}\n",
					  "Updated: ${date}01\n";
			close F;
			$built{$cust_id}{$netdir} = 1;
		}else{
			warn "cannot write to ${homedir}/net-${netdir}/data/org/${cust_id}.txt.tmp: $!\n";
		}
	}

	# build network
	open(F, ">${homedir}/net-${netdir}/data/network/${netblock}-${cidr}.txt.tmp") ||
	 die "cannot write to ${homedir}/net-${netdir}/data/network/${netblock}-${cidr}.txt.tmp: $!\n";
	my $broadcast = $ipmath->get_broadcast($netblock,$cidr);
	print F "ID: NETBLCK-${ucour_name}.$netblocks{$gpid{$parent_id}}\n",
	        "Auth-Area: $netblocks{$gpid{$parent_id}}\n",
	        "Network-Name: $ucname-${netblock}\n",
	        "IP-Network: ${netblock}/${cidr}\n",
	        "IP-Network-Block: ${netblock} - ${broadcast}\n",
	        "Organization: ";
	if($row->{private_id}){
		print F "$cfg{our_name}: $row->{private_id}\n",
		        "Tech-Contact: prwhois-$row->{private_id}\@$cfg{our_domain}\n";
	}else{
		print F "$cust->[1]\n",
		        "Tech-Contact: $row->{tech_email}\n";
	}
	print F "Admin-Contact: $cfg{rwhois_nic_handle}\n",
	        "Created: ${created}\n",
		     "Updated: ${date}01\n",
	        "Updated-By: $cfg{rwhois_hostmaster}\n";
	close F;

}

$d_dbh->disconnect();

# we do this here, because we want this process to be as quick as possible
# replacing one-at-a-time in the loop above is far too slow and causes rwhoisd to quit)
foreach my $dir (@directories){
	warn "moving over files in $dir..\n" if($DEBUG);
	foreach my $subdir ('network','org'){
		opendir(D, "${homedir}/net-${dir}/data/${subdir}/") || die "Cannot opendir $dir/data/$subdir: $!\n";
		foreach my $file (readdir D){
			if($file =~ /(.+)\.tmp$/){
				my $new = $1;
				rename("${homedir}/net-${dir}/data/${subdir}/${file}","${homedir}/net-${dir}/data/${subdir}/${new}") ||
				  warn "cannot rename ${homedir}/net-${dir}/data/${subdir}/${file}: $!\n";
			}
		}
		closedir D;
		# nuke stale files
		opendir(D, "${homedir}/net-${dir}/data/${subdir}/") || die "Cannot opendir $dir/data/$subdir: $!\n";
		foreach my $file (grep {/txt/} readdir D){
			my $mtime = (stat("${homedir}/net-${dir}/data/${subdir}/${file}"))[9];
			my $delta = ($now - $mtime);
			if($delta > 691200){
				warn "unlinking $dir/data/$subdir/$file\n" if($DEBUG);
				unlink("${homedir}/net-${dir}/data/${subdir}/${file}") || warn "couldnt unlink $dir/data/$subdir/$file: $!\n";
			}
		}
		closedir D;
	}
}

rename("${homedir}/rwhoisd.auth_area.tmp","${homedir}/rwhoisd.auth_area") || die "could not rename auth_area: $!\n";

# reindex
warn "reindexing..\n" if($DEBUG);
chop(my $output = `${homedir}/bin/rwhois_indexer -c ${homedir}/rwhoisd.conf -i -s txt 2>/dev/null`);
die "could not index\n" unless($output =~ /done\.$/);

if($restart){
	warn "restarting..\n" if($DEBUG);
	system('svc','-du','/service/rwhoisd');
}
