#!/usr/local/bin/perl

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use CGI qw(:standard -nosticky);
use CGI::Carp ('fatalsToBrowser');
use DBI;
use HTML::Template;
use IPAnalyze::Auth;
use IPAnalyze::Misc;

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/houston.conf') || die "cannot open houston.conf: $!\n";
while(<CONFIG>){
   s/^(.*)\s+#.*/$1/;
   next if(/^(?:#.*)?$/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;


my $q = CGI->new();
my $auth = IPAnalyze::Auth->new();
my $misc = IPAnalyze::Misc->new();
my $script = $ENV{'SCRIPT_NAME'};
my $url = 'http://' . $ENV{'HTTP_HOST'} . $script;

my $dbh;
until($dbh){
	eval {
		local $SIG{ALRM} = sub { die "DBconnect timeout.\n"; };
		alarm(10);
		$dbh = DBI->connect($cfg{dsn}, $cfg{dbun}, $cfg{dbpw}, {RaiseError => 1});
		alarm(0);
	};
	alarm(0);
	warn "$DBI::errstr\n" if($DBI::errstr);
	if($@){
		print "error: $@\n";
	}
	sleep 1;
}
my $t = $auth->manage_cookie($dbh,$q,$misc);

if($t == 2){
	# header already printed in manage_cookie
	show_login();
	exit;
}elsif($q->param('login')){
	if(my @errors = $auth->check_credentials($dbh,$q)){
		print $q->header(-expires=>'now');
		show_login();
		foreach my $error (@errors){
			print "<p align=\"center\"><i><font color=\"#FF0000\"><b>$error</b></font></i></p>";
		}
		exit;
	}
	if(my $dest = $q->param('dest')){
		my $redirect = 'http://' . $ENV{'HTTP_HOST'} . $dest;
		print $q->redirect($redirect);
	}else{
		print $q->redirect($url);
	}
	exit;
}elsif($q->param('logout')){
	$auth->logoff_ap($script,$q);
	exit;
}

# show main page

print $q->header(-expires=>'now');

my $username = $auth->get_username($dbh,$q);
unless($username){
	show_login();
	exit;
}


my $template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
$template->param(SCRIPT => $script);
$template->param(APP => 'IPAnalyze');
$template->param(DESC => 'Mission Control GUI');
$template->param(VERSION => $cfg{version});
$template->param(USERNAME => $username);
$template->param(SHOW_HOUSTON => 1);

print $template->output;

sub show_login {
	my $self = shift;

	my $template = HTML::Template->new(filename => "$cfg{html_dir}/login.html");
	$template->param(SCRIPT => $script);
	$template->param(APP => 'IPAnalyze Login');
	$template->param(VERSION => $cfg{version});
	$template->param(DEST => $q->param('dest')) if($q->param('dest'));
	
	print $template->output;
}
